@@ -75,6 +75,7 @@
 #include "gnc-icons.h"
 #include "print-session.h"
 
+#include <unordered_map>
 #include <memory>
 #include <gnc-report.h>
 
@@ -84,13 +85,10 @@ a gnucash- prefix will do. Then just set a log level
 with qof_log_set_level().*/
 static QofLogModule log_module = GNC_MOD_GUI;
 
-static GObjectClass *parent_class = nullptr;
-
 // A static GHashTable to record the usage count for each printer
-// output name. FIXME: Currently this isn't cleaned up at program
-// shutdown because there isn't a place to easily insert a finalize()
-// function for this. Oh well.
-static GHashTable *static_report_printnames = nullptr;
+// output name.
+
+static std::unordered_map<std::string,unsigned> static_report_printnames;
 
 // Property-id values.
 enum
@@ -140,8 +138,6 @@ G_DEFINE_TYPE_WITH_PRIVATE(GncPluginPageReport, gnc_plugin_page_report, GNC_TYPE
 #define GNC_PLUGIN_PAGE_REPORT_GET_PRIVATE(o)  \
    ((GncPluginPageReportPrivate*)gnc_plugin_page_report_get_instance_private((GncPluginPageReport*)o))
 
-static void gnc_plugin_page_report_class_init( GncPluginPageReportClass *klass );
-static void gnc_plugin_page_report_init( GncPluginPageReport *plugin_page );
 static GObject *gnc_plugin_page_report_constructor(GType this_type, guint n_properties, GObjectConstructParam *properties);
 static void gnc_plugin_page_report_finalize (GObject *object);
 static void gnc_plugin_page_report_setup( GncPluginPage *ppage );
@@ -343,8 +339,6 @@ gnc_plugin_page_report_class_init (GncPluginPageReportClass *klass)
     GObjectClass *object_class = G_OBJECT_CLASS (klass);
     GncPluginPageClass *gnc_plugin_page_class = GNC_PLUGIN_PAGE_CLASS(klass);
 
-    parent_class = static_cast<GObjectClass*>(g_type_class_peek_parent (klass));
-
     object_class->constructor = gnc_plugin_page_report_constructor;
     object_class->finalize = gnc_plugin_page_report_finalize;
 
@@ -372,11 +366,6 @@ gnc_plugin_page_report_class_init (GncPluginPageReportClass *klass)
                                              _("The numeric ID of the report."),
                                                        -1, G_MAXINT, -1,
                                                        paramspec));
-
-    // Also initialize the report name usage count table
-    if (!static_report_printnames)
-        static_report_printnames = g_hash_table_new_full(g_str_hash,
-                                   g_str_equal, g_free, nullptr);
 }
 
 static void
@@ -385,7 +374,7 @@ gnc_plugin_page_report_finalize (GObject *object)
     g_return_if_fail (GNC_IS_PLUGIN_PAGE_REPORT (object));
 
     ENTER("object %p", object);
-    G_OBJECT_CLASS (parent_class)->finalize (object);
+    G_OBJECT_CLASS (gnc_plugin_page_report_parent_class)->finalize (object);
     LEAVE(" ");
 }
 
@@ -1199,16 +1188,9 @@ gnc_plugin_page_report_init ( GncPluginPageReport *plugin_page )
 static GObject*
 gnc_plugin_page_report_constructor(GType this_type, guint n_properties, GObjectConstructParam *properties)
 {
-    GObject *obj;
-    GncPluginPageReportClass *our_class;
-    GObjectClass *parent_class;
+    GObject *obj = G_OBJECT_CLASS (gnc_plugin_page_report_parent_class)->constructor(this_type, n_properties, properties);
+
     gint reportId = -42;
-
-    our_class = GNC_PLUGIN_PAGE_REPORT_CLASS (
-                    g_type_class_peek (GNC_TYPE_PLUGIN_PAGE_REPORT));
-    parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (our_class));
-    obj = parent_class->constructor(this_type, n_properties, properties);
-
     for (decltype(n_properties) i = 0; i < n_properties; i++)
     {
         GObjectConstructParam prop = properties[i];
@@ -1960,32 +1942,16 @@ static gchar *report_create_jobname(GncPluginPageReportPrivate *priv)
     {
         /* And one final checking issue: We want to avoid allocating
          * the same name twice for a saved PDF.  Hence, we keep a
-         * GHashTable with the usage count of existing output
-         * names. (Because I'm lazy, I just use a static GHashTable
-         * for this.) */
-        gpointer value;
-        gboolean already_found;
-        g_assert(static_report_printnames);
+         * unordered_map with the usage count of existing output
+         * names. */
 
-        // Lookup the existing usage count
-        value = g_hash_table_lookup(static_report_printnames, job_name);
-        already_found = (value != nullptr);
-        if (!value)
-        {
-            value = GINT_TO_POINTER(0);
-        }
+        auto& value = static_report_printnames[job_name];
+        value++;
 
-        // Increment the stored usage count
-        value = GINT_TO_POINTER(1 + GPOINTER_TO_INT(value));
-        // and store it again
-        g_hash_table_insert(static_report_printnames, g_strdup(job_name), value);
-
-        // If the previous usage count was more than 0, append the current
-        // count (which is now 2 or higher) to the resulting name
-        if (already_found)
+        if (value > 1)
         {
             // The name was already in use, so modify the name again
-            gchar *tmp = g_strdup_printf("%s_%d", job_name, (int) GPOINTER_TO_INT(value));
+            gchar *tmp = g_strdup_printf("%s_%d", job_name, value);
             g_free(job_name);
             job_name = tmp;
         }
