@@ -35,6 +35,9 @@
 #endif
 #include <boost/algorithm/string.hpp>
 #include <boost/filesystem.hpp>
+#ifdef BOOST_WINDOWS_API
+#include <boost/process/windows.hpp>
+#endif
 #include <boost/process.hpp>
 #include <boost/regex.hpp>
 #include <boost/property_tree/ptree.hpp>
@@ -219,12 +222,18 @@ GncFQQuoteSource::run_cmd (const StrVec& args, const std::string& json_string) c
 				bp::std_out > out_buf,
 				bp::std_err > err_buf,
 				bp::std_in < input_buf,
+#ifdef BOOST_WINDOWS_API
+                                bp::windows::create_no_window,
+#endif
 				svc);
 	else
 	    process = bp::child(c_cmd, args,
 				bp::std_out > out_buf,
 				bp::std_err > err_buf,
 				bp::std_in < input_buf,
+#ifdef BOOST_WINDOWS_API
+                                bp::windows::create_no_window,
+#endif
 				bp::env["ALPHAVANTAGE_API_KEY"] = m_api_key,
 				svc);
 
@@ -567,7 +576,7 @@ parse_quote_json(PriceParams& p, const bpt::ptree& comm_pt)
 
 
     PINFO("Commodity: %s", p.mnemonic);
-    PINFO("  Success: %s", (inverted ? "yes" : "no"));
+    PINFO("  Success: %s", (p.success ? "yes" : "no"));
     PINFO("     Date: %s", (p.date ? p.date->c_str() : "missing"));
     PINFO("     Time: %s", (p.time ? p.time->c_str() : "missing"));
     PINFO(" Currency: %s", (p.currency ? p.currency->c_str() : "missing"));
@@ -584,34 +593,27 @@ calc_price_time(const PriceParams& p)
      * as gnucash insists on having a valid format". It's also wrong,
      * as it lacks seconds. Best ignored.
      */
-    if (p.date)
+    if (p.date && !p.date->empty())
     {
-        /* Returned date is always in MM/DD/YYYY format according to
-         * F::Q man page, transform it to simplify conversion to
-         * GncDateTime.
-         */
-        auto date_tmp = *p.date;
-        auto iso_date_str = date_tmp.substr (6, 4) + "-" + date_tmp.substr (0, 2) +
-            "-" + date_tmp.substr (3, 2);
         try
         {
-            auto close_time{GncDateTime(iso_date_str + " 16:00:00")};
+            auto quote_time{GncDateTime(GncDate(*p.date, "m-d-y"))};
             PINFO("Quote date included, using %s for %s:%s",
-                  close_time.format("%Y-%m-%d %H:%M:%S").c_str(), p.ns, p.mnemonic);
-            return static_cast<time64>(close_time);
-        }
-        catch (...)
+                  quote_time.format("%Y-%m-%d %H:%M:%S %z").c_str(), p.ns, p.mnemonic);
+            return static_cast<time64>(quote_time);
+          }
+        catch (const std::exception &err)
         {
             auto now{GncDateTime()};
-            PWARN("Warning: failed to parse quote date '%s' for %s:%s - will use %s",
-                  iso_date_str.c_str(),  p.ns, p.mnemonic, now.format("%Y-%m-%d %H:%M%S").c_str());
+            PWARN("Warning: failed to parse quote date '%s' for %s:%s because %s - will use %s",
+                  p.date->c_str(),  p.ns, p.mnemonic, err.what(), now.format("%Y-%m-%d %H:%M:%S %z").c_str());
             return static_cast<time64>(now);
         }
     }
 
     auto now{GncDateTime()};
     PINFO("No date  was returned for %s:%s - will use %s",
-          p.ns, p.mnemonic, now.format("%Y-%m-%d %H:%M%S").c_str());
+          p.ns, p.mnemonic, now.format("%Y-%m-%d %H:%M:%S %z").c_str());
     return static_cast<time64>(now);
 }
 
@@ -735,7 +737,7 @@ GncQuotesImpl::parse_quotes (const std::string& quote_str)
 {
     bpt::ptree pt;
     std::istringstream ss {quote_str};
-    const char* what = nullptr;
+    std::string what;
 
     try
     {
@@ -754,15 +756,26 @@ GncQuotesImpl::parse_quotes (const std::string& quote_str)
     }
     catch (...) {
         std::string error_msg{_("Failed to parse result returned by Finance::Quote.")};
+        error_msg += "\n";
+        //Translators: This labels the return value of a query to Finance::Quote written in an error.
+        error_msg += _("Result:");
+        error_msg += "\n";
+        error_msg += quote_str;
         throw(GncQuoteException(error_msg));
     }
-    if (what)
+    if (!what.empty())
     {
         std::string error_msg{_("Failed to parse result returned by Finance::Quote.")};
         error_msg += "\n";
+        //Translators: This is the error message reported by the Online Quotes processing code.
         error_msg += _("Error message:");
         error_msg += "\n";
         error_msg += what;
+        error_msg += "\n";
+        //Translators: This labels the return value of a query to Finance::Quote written in an error.
+        error_msg += _("Result:");
+        error_msg += "\n";
+        error_msg += quote_str;
         throw(GncQuoteException(error_msg));
     }
     return pt;
