@@ -72,6 +72,18 @@ typedef struct _SxTxnCreationData
     GList **creation_errors;
 } SxTxnCreationData;
 
+struct _GncSxInstanceModel
+{
+    GObject parent;
+    gboolean disposed;
+
+    gint qof_event_handler_id;
+
+    GDate range_end;
+    gboolean include_disabled;
+    GList *sx_instance_list; /* <GncSxInstances*> */
+};
+
 static GncSxInstanceModel* gnc_sx_instance_model_new(void);
 
 static GncSxInstance* gnc_sx_instance_new(GncSxInstances *parent, GncSxInstanceState state, GDate *date, void *temporal_state, gint sequence_num);
@@ -90,6 +102,14 @@ typedef struct
     gnc_numeric amount;
 } ScrubItem;
 
+enum
+{
+    REMOVING, UPDATED, ADDED,
+    LAST_SIGNAL
+};
+
+static guint signals[LAST_SIGNAL] = { 0 };
+
 static void
 scrub_sx_split_numeric (Split* split, gboolean is_credit, GList **changes)
 {
@@ -348,6 +368,7 @@ _get_vars_helper(Transaction *txn, void *var_hash_data)
             var_name = var_name_from_commodities(split_cmdty, txn_cmdty);
             var = gnc_sx_variable_new(var_name);
             g_hash_table_insert(var_hash, g_strdup(var->name), var);
+            g_free (var_name);
         }
     }
 
@@ -670,7 +691,7 @@ gnc_sx_instance_model_class_init (GncSxInstanceModelClass *klass)
     object_class->dispose = gnc_sx_instance_model_dispose;
     object_class->finalize = gnc_sx_instance_model_finalize;
 
-    klass->removing_signal_id =
+    signals[REMOVING] =
         g_signal_new("removing",
                      GNC_TYPE_SX_INSTANCE_MODEL,
                      G_SIGNAL_RUN_FIRST,
@@ -682,7 +703,7 @@ gnc_sx_instance_model_class_init (GncSxInstanceModelClass *klass)
                      1,
                      G_TYPE_POINTER);
 
-    klass->updated_signal_id =
+    signals[UPDATED] =
         g_signal_new("updated",
                      GNC_TYPE_SX_INSTANCE_MODEL,
                      G_SIGNAL_RUN_FIRST,
@@ -694,7 +715,7 @@ gnc_sx_instance_model_class_init (GncSxInstanceModelClass *klass)
                      1,
                      G_TYPE_POINTER);
 
-    klass->added_signal_id =
+    signals[ADDED] =
         g_signal_new("added",
                      GNC_TYPE_SX_INSTANCE_MODEL,
                      G_SIGNAL_RUN_FIRST,
@@ -1141,7 +1162,7 @@ split_apply_exchange_rate (Split *split, GHashTable *bindings,
     if (exchange_rate_var != NULL)
     {
         exchange_rate = exchange_rate_var->value;
-        DEBUG("exchange_rate is %s", gnc_numeric_to_string (exchange_rate));
+        DEBUG("exchange_rate is %s", gnc_num_dbg_to_string (exchange_rate));
     }
     g_free (exchange_rate_var_name);
 
@@ -1154,7 +1175,7 @@ split_apply_exchange_rate (Split *split, GHashTable *bindings,
                               GNC_HOW_RND_ROUND_HALF_UP);
 
 
-    DEBUG("amount is %s for memo split '%s'", gnc_numeric_to_string (amt),
+    DEBUG("amount is %s for memo split '%s'", gnc_num_dbg_to_string (amt),
             xaccSplitGetMemo (split));
     xaccSplitSetAmount(split, amt); /* marks split dirty */
 
@@ -1318,7 +1339,7 @@ create_each_transaction_helper(Transaction *template_txn, void *user_data)
                                                      creation_data);
             xaccSplitSetValue(copying_split, final);
             DEBUG("value is %s for memo split '%s'",
-                    gnc_numeric_to_string (final),
+                    gnc_num_dbg_to_string (final),
                     xaccSplitGetMemo (copying_split));
             if (! gnc_commodity_equal(split_cmdty, txn_cmdty))
             {
@@ -1898,3 +1919,8 @@ GHashTable* gnc_sx_all_instantiate_cashflow_all(GDate range_start, GDate range_e
                                     result_map, NULL);
     return result_map;
 }
+
+GList *gnc_sx_instance_model_get_sx_instances_list (GncSxInstanceModel *model)
+{
+    return model->sx_instance_list;
+}
