@@ -362,10 +362,6 @@ test_price_list_is_duplicate (Fixture *fixture, gconstpointer pData)
 gboolean
 gnc_price_list_insert(PriceList **prices, GNCPrice *p, gboolean check_dupl)// Local: 3:0:0
 */
-/* static void
-test_gnc_price_list_insert (Fixture *fixture, gconstpointer pData)
-{
-}*/
 /* gnc_price_list_remove
 gboolean
 gnc_price_list_remove(PriceList **prices, GNCPrice *p)// Local: 1:0:0
@@ -582,7 +578,6 @@ static void
 setup(PriceDBFixture *fixture, gconstpointer data)
 {
     QofBook *book = NULL;
-    GNCPrice *price = NULL;
     gnc_pricedb_register();
     book = qof_book_new();
     fixture->com = setup_commodities(book);
@@ -683,6 +678,37 @@ guint
 gnc_pricedb_get_num_prices(GNCPriceDB *db)// C: 2 in 1  Local: 0:0:0
 */
 
+static void
+test_gnc_price_list_insert (PriceDBFixture *fixture, gconstpointer pData)
+{
+    GNCPriceDB *db = fixture->pricedb;
+    Commodities *c = fixture->com;
+    QofBook *book = qof_instance_get_book(QOF_INSTANCE(db));
+
+    g_assert_cmpint (gnc_pricedb_get_num_prices(fixture->pricedb), ==, 42);
+
+    // nop because 12/4/09 already exists
+    gnc_pricedb_add_price (db, construct_price(book, c->usd, c->aud,
+                                               gnc_dmy2time64(12, 4, 2009),
+                                               PRICE_SOURCE_USER_PRICE,
+                                               gnc_numeric_create(131190, 10000)));
+    g_assert_cmpint (gnc_pricedb_get_num_prices(fixture->pricedb), ==, 42);
+
+    // num_prices increases because because 14/4/09 doesn't exist yet
+    gnc_pricedb_add_price (db, construct_price(book, c->usd, c->aud,
+                                               gnc_dmy2time64(14, 4, 2009),
+                                               PRICE_SOURCE_USER_PRICE,
+                                               gnc_numeric_create(131190, 10000)));
+    g_assert_cmpint (gnc_pricedb_get_num_prices(fixture->pricedb), ==, 43);
+
+    // nop because because 14/4/09 already exists
+    gnc_pricedb_add_price (db, construct_price(book, c->usd, c->aud,
+                                               gnc_dmy2time64(14, 4, 2009),
+                                               PRICE_SOURCE_USER_PRICE,
+                                               gnc_numeric_create(131190, 10000)));
+    g_assert_cmpint (gnc_pricedb_get_num_prices(fixture->pricedb), ==, 43);
+}
+
 static void
 test_gnc_price_list_equal (PriceDBFixture *fixture, gconstpointer pData)
 {
@@ -692,23 +718,23 @@ test_gnc_price_list_equal (PriceDBFixture *fixture, gconstpointer pData)
     gnc_commodity *aud = c->aud;
 
     /* p1 and p2 are both NULL */
-    g_assert (gnc_price_list_equal (p1, p2));
+    g_assert_true (gnc_price_list_equal (p1, p2));
 
     p1 = gnc_pricedb_lookup_latest_any_currency(fixture->pricedb, aud);
 
     /* p1 is PriceList, p2 is NULL */
-    g_assert (!gnc_price_list_equal (p1, p2));
+    g_assert_true (!gnc_price_list_equal (p1, p2));
 
     p2 = p1;
 
     /* p1 and p2 both point to the same PriceList */
-    g_assert (gnc_price_list_equal (p1, p2));
+    g_assert_true (gnc_price_list_equal (p1, p2));
 
     p2 = gnc_pricedb_lookup_latest_any_currency(fixture->pricedb, aud);
 
     /* p1 and p2 are different PriceLists, but are identical in contents */
-    g_assert (p1 != p2);
-    g_assert (gnc_price_list_equal (p1, p2));
+    g_assert_true (p1 != p2);
+    g_assert_true (gnc_price_list_equal (p1, p2));
 
     gnc_price_list_destroy (p1);
     gnc_price_list_destroy (p2);
@@ -832,7 +858,7 @@ static void test_gnc_pricedb_remove_old_prices (PriceDBFixture *fixture, gconstp
 
     g_assert_cmpint (gnc_pricedb_get_num_prices(fixture->pricedb), ==, 42);
 
-    g_assert (gnc_pricedb_remove_old_prices(fixture->pricedb, comm_list,
+    g_assert_true (gnc_pricedb_remove_old_prices(fixture->pricedb, comm_list,
                                            fiscal_end_date, t_cut1,
                                            PRICE_REMOVE_SOURCE_USER, // source is USER
                                            PRICE_REMOVE_KEEP_NONE)); // keep none
@@ -840,35 +866,35 @@ static void test_gnc_pricedb_remove_old_prices (PriceDBFixture *fixture, gconstp
     g_assert_cmpint (gnc_pricedb_get_num_prices(fixture->pricedb), ==, 39);
 
     // there should be no prices before cutoff, returns false
-    g_assert (!gnc_pricedb_remove_old_prices(fixture->pricedb, comm_list,
+    g_assert_true (!gnc_pricedb_remove_old_prices(fixture->pricedb, comm_list,
                                            NULL, t_cut,
                                            PRICE_REMOVE_SOURCE_FQ,   // source is FQ
                                            PRICE_REMOVE_KEEP_NONE)); // keep none
 
     g_assert_cmpint (gnc_pricedb_get_num_prices(fixture->pricedb), ==, 39);
 
-    g_assert (gnc_pricedb_remove_old_prices(fixture->pricedb, comm_list,
+    g_assert_true (gnc_pricedb_remove_old_prices(fixture->pricedb, comm_list,
                                            fiscal_end_date, t_cut1,
                                            source_all,                      // source is ALL
                                            PRICE_REMOVE_KEEP_LAST_WEEKLY)); // keep last of week
 
     g_assert_cmpint (gnc_pricedb_get_num_prices(fixture->pricedb), ==, 38);
 
-    g_assert (gnc_pricedb_remove_old_prices(fixture->pricedb, comm_list,
+    g_assert_true (gnc_pricedb_remove_old_prices(fixture->pricedb, comm_list,
                                            fiscal_end_date, t_cut2,
                                            PRICE_REMOVE_SOURCE_FQ,           // source is FQ
                                            PRICE_REMOVE_KEEP_LAST_MONTHLY)); // keep last of month
 
     g_assert_cmpint (gnc_pricedb_get_num_prices(fixture->pricedb), ==, 37);
 
-    g_assert (gnc_pricedb_remove_old_prices(fixture->pricedb, comm_list,
+    g_assert_true (gnc_pricedb_remove_old_prices(fixture->pricedb, comm_list,
                                            fiscal_end_date, t_cut2,
                                            source_all,                         // source is all
                                            PRICE_REMOVE_KEEP_LAST_QUARTERLY)); // keep last of quarter
 
     g_assert_cmpint (gnc_pricedb_get_num_prices(fixture->pricedb), ==, 35);
 
-    g_assert (gnc_pricedb_remove_old_prices(fixture->pricedb, comm_list,
+    g_assert_true (gnc_pricedb_remove_old_prices(fixture->pricedb, comm_list,
                                            fiscal_end_date, t_cut2,
                                            source_all,                      // source is all
                                            PRICE_REMOVE_KEEP_LAST_PERIOD)); // keep last of period
@@ -906,12 +932,12 @@ test_gnc_pricedb_lookup_latest (PriceDBFixture *fixture, gconstpointer pData)
                                                          fixture->com->eur);
     time64 t = gnc_dmy2time64(12, 11, 2014);
     time64 price_time = gnc_price_get_time64(price);
-    g_assert(gnc_price_get_commodity(price) == fixture->com->gbp);
-    g_assert(gnc_price_get_currency(price) == fixture->com->eur);
+    g_assert_true(gnc_price_get_commodity(price) == fixture->com->gbp);
+    g_assert_true(gnc_price_get_currency(price) == fixture->com->eur);
     g_assert_cmpint(price_time, ==, t);
     price2 = gnc_pricedb_lookup_latest(fixture->pricedb, fixture->com->eur,
                                       fixture->com->gbp);
-    g_assert(price2 == price);
+    g_assert_true(price2 == price);
     gnc_price_unref(price);
     gnc_price_unref(price2);
 }
@@ -1074,9 +1100,9 @@ gnc_pricedb_has_prices(GNCPriceDB *db,// C: 4 in 3 SCM: 1  Local: 0:0:0
 static void
 test_gnc_pricedb_has_prices (PriceDBFixture *fixture, gconstpointer pData)
 {
-    g_assert(gnc_pricedb_has_prices(fixture->pricedb, fixture->com->usd,
+    g_assert_true(gnc_pricedb_has_prices(fixture->pricedb, fixture->com->usd,
                                     fixture->com->dkk));
-    g_assert(!gnc_pricedb_has_prices(fixture->pricedb, fixture->com->usd,
+    g_assert_true(!gnc_pricedb_has_prices(fixture->pricedb, fixture->com->usd,
                                      fixture->com->gbp));
 }
 /* gnc_pricedb_get_prices
@@ -1119,7 +1145,7 @@ test_gnc_pricedb_lookup_day_t64 (PriceDBFixture *fixture, gconstpointer pData)
     price = gnc_pricedb_lookup_day_t64(fixture->pricedb,
                                    fixture->com->usd,
                                    fixture->com->gbp, t);
-    g_assert(price == NULL);
+    g_assert_true(price == NULL);
     g_log_set_default_handler (hdlr, 0);
 }
 
@@ -1467,6 +1493,45 @@ test_gnc_pricedb_get_nearest_before_price (PriceDBFixture *fixture, gconstpointe
     g_assert_cmpint(result.num, ==, 278150);
     g_assert_cmpint(result.denom, ==, 1331);
 }
+
+/* gnc_pricedb_foreach_price
+gboolean
+gnc_pricedb_foreach_price(GNCPriceDB *db,// C: 2 in 2  Local: 6:0:0
+*/
+
+static void prepend_price_time64 (GNCPrice *p, GList **lst)
+{
+    *lst = g_list_prepend (*lst, GUINT_TO_POINTER(gnc_price_get_time64 (p)));
+}
+
+static void inc_counter (GNCPrice *p, guint *count)
+{
+    (*count)++;
+}
+
+static void
+test_gnc_pricedb_foreach_price (PriceDBFixture *fixture, gconstpointer pData)
+{
+    /* unstable -- cannot guarantee order. count number prices. */
+    guint count = 0;
+    gnc_pricedb_foreach_price (fixture->pricedb, (GncPriceForeachFunc)inc_counter, &count, FALSE);
+    g_assert_cmpint (count, ==, 42);
+
+    /* stable -- can guarantee order. check price dates. */
+    GList *lst = NULL;
+    gnc_pricedb_foreach_price (fixture->pricedb, (GncPriceForeachFunc)prepend_price_time64, &lst, TRUE);
+
+    gchar *date = qof_print_date((time64)g_list_first (lst)->data);
+    g_assert_cmpstr (date, ==, "04/11/09");
+    g_free (date);
+
+    date = qof_print_date((time64)g_list_last (lst)->data);
+    g_assert_cmpstr (date, ==, "11/12/14");
+    g_free (date);
+
+    g_list_free (lst);
+}
+
 /* pricedb_foreach_pricelist
 static void
 pricedb_foreach_pricelist(gpointer key, gpointer val, gpointer user_data)// Local: 0:1:0
@@ -1507,14 +1572,6 @@ stable_price_traversal(GNCPriceDB *db,// Local: 1:0:0
 test_stable_price_traversal (Fixture *fixture, gconstpointer pData)
 {
 }*/
-/* gnc_pricedb_foreach_price
-gboolean
-gnc_pricedb_foreach_price(GNCPriceDB *db,// C: 2 in 2  Local: 6:0:0
-*/
-/* static void
-test_gnc_pricedb_foreach_price (Fixture *fixture, gconstpointer pData)
-{
-}*/
 /* add_price_to_list
 static gboolean
 add_price_to_list (GNCPrice *p, gpointer data)// Local: 0:1:0
@@ -1673,7 +1730,7 @@ test_suite_gnc_pricedb (void)
 // GNC_TEST_ADD (suitename, "gnc price equal", Fixture, NULL, setup, test_gnc_price_equal, teardown);
 // GNC_TEST_ADD (suitename, "compare prices by date", Fixture, NULL, setup, test_compare_prices_by_date, teardown);
 // GNC_TEST_ADD (suitename, "price list is duplicate", Fixture, NULL, setup, test_price_list_is_duplicate, teardown);
-// GNC_TEST_ADD (suitename, "gnc price list insert", Fixture, NULL, setup, test_gnc_price_list_insert, teardown);
+GNC_TEST_ADD (suitename, "gnc price list insert", PriceDBFixture, NULL, setup, test_gnc_price_list_insert, teardown);
 // GNC_TEST_ADD (suitename, "gnc price list remove", Fixture, NULL, setup, test_gnc_price_list_remove, teardown);
 // GNC_TEST_ADD (suitename, "price list destroy helper", Fixture, NULL, setup, test_price_list_destroy_helper, teardown);
 // GNC_TEST_ADD (suitename, "gnc price list destroy", Fixture, NULL, setup, test_gnc_price_list_destroy, teardown);
@@ -1733,7 +1790,7 @@ GNC_TEST_ADD (suitename, "gnc price list equal", PriceDBFixture, NULL, setup, te
 // GNC_TEST_ADD (suitename, "unstable price traversal", Fixture, NULL, setup, test_unstable_price_traversal, teardown);
 // GNC_TEST_ADD (suitename, "compare kvpairs by commodity key", Fixture, NULL, setup, test_compare_kvpairs_by_commodity_key, teardown);
 // GNC_TEST_ADD (suitename, "stable price traversal", Fixture, NULL, setup, test_stable_price_traversal, teardown);
-// GNC_TEST_ADD (suitename, "gnc pricedb foreach price", Fixture, NULL, setup, test_gnc_pricedb_foreach_price, teardown);
+GNC_TEST_ADD (suitename, "gnc pricedb foreach price", PriceDBFixture, NULL, setup, test_gnc_pricedb_foreach_price, teardown);
 // GNC_TEST_ADD (suitename, "add price to list", Fixture, NULL, setup, test_add_price_to_list, teardown);
 // GNC_TEST_ADD (suitename, "gnc price fixup legacy commods", Fixture, NULL, setup, test_gnc_price_fixup_legacy_commods, teardown);
 // GNC_TEST_ADD (suitename, "gnc price print", Fixture, NULL, setup, test_gnc_price_print, teardown);
